#ifndef CUTGRAPH_H
#define CUTGRAPH_H

#include <iostream>
#include <set>
#include <queue>
#include <map>
#include <vector>
#include <utility>
#include <fstream>
#include "MeshLib\Mesh.h"
#include "MeshLib\Iterators.h"

using namespace  std;
class CutGraph{
public:
	CutGraph();
	CutGraph(Mesh *mesh);
	~CutGraph();

	void ComputeDualGraph();
	void ComputeSpanningTree();
	void TopoCut();
	void TrimDanglingEdge(char *str);
	int FindWid(int old_vid, int fid);
	void ComputeFundamentalDomain(char *ofilename);
private:
	Mesh *cmesh;

	map<int, int> fid2vid_dual; //mapping from fid to vid on the dual graph
	map<int, int> vid_dual2fid;

	vector<vector<int> > AdjList_dual; // Adjacency List for the dual graph
	vector<pair<int, int> > edgelist_dual;// edgelist on cmesh e=(fid0, fid1) -> on dual graph e=(vid0, vid1)
	vector<pair<int, int> > edgelist_span_tree;

	vector<vector<int> > AdjList_dual_stree; //Adjacency List for the spanning tree of dual graph
	vector<vector<int> > AdjList_topocut; //Adjacency List for the topological cut at first, and then the cut graph

	vector<bool> vid4CutGraph; //if a vertex is on the cut graph, then it is marked in this vector
	vector<int> vidOnCutGraph; //vertices on the cut graph

	vector<vector<HalfEdge *> > wedges; //vector of wedges, its index is the wedge ID 
	vector<vector<int> > wid2fids; //wedge ID to face ID
	vector<vector<int> > old_vid2wids; // old_vidv-1 to its related wedge IDs corresponding to vector of wedges
	vector<int> wid2old_vid; // wedge ID to related old_vid
	vector<int> wid2new_vid; // wedge ID to new_vertex ID
};

#endif