#include <iostream>
#include <string>
#include <sstream>
#include <fstream>
#include <queue>
#include <assert.h>
#include "Face.h"
#include "Vertex.h"
#include "VertexNode.h"

using namespace std;

void Swap(Vertex *&a, Vertex *&b)
{
	Vertex *temp = a;
	a = b;
	b = temp;
}

void AddVnode(Vertex *v0, Vertex *v1, vector<vector<VertexNode*> > &vnodeList, Face* f){
	/* Add the edge(v0, v1) into vnodeList and associate it to the corresponding face
		since v0's 1-ring neighbor is a constant, the computational complexity of 
		this method if O(1);
	*/
	vector<VertexNode *> &vvnode =  vnodeList[v0->vid]; 
	int i=0;
	for (;i<vvnode.size();i++){
		if(vvnode[i]->v->vid == v1->vid)
			break;
	}// find edge(v0, v1)
	if(i<vvnode.size()){ //if edge(v0, v1) exists, count its appearance times
		int idx = vvnode[i]->count;
		vvnode[i]->f[idx] = f; // associate this edge to the face
		vvnode[i]->count++;
	}else{// if edge(v0, v1) does not exist, insert it.
		VertexNode *vn = new VertexNode;
		vn->v = v1;
		vn->f[0] = f;
		vn->count = 1;
		vvnode.push_back(vn);
	}
}
void DeleteVnode(Vertex *v0, Vertex *v1, vector<vector<VertexNode*> > &vnodeList, Face* f){
	/*Delete edge(v0, v1) from the the vnodeList and release the correspondence between
		f and this edge, since we want to correct it .
		the complexity is the same with AddVnode, O(1)
	*/
	vector<VertexNode *> &vvnode =  vnodeList[v0->vid];
	int i=0;
	for (;i<vvnode.size();i++){
		if(vvnode[i]->v->vid == v1->vid)
			break;
	}//find edge(v0, v1)
	switch(vvnode[i]->count){
	case 2: //if there is two edge(v0, v1)'s, it should a contradiction
		if(vvnode[i]->f[0]->fid==f->fid){
			vvnode[i]->f[0] = NULL;
		}else if(vvnode[i]->f[1]->fid==f->fid){
			vvnode[i]->f[1] = NULL;
		}
		break;
	case 1:
		vvnode[i]->f[0] = NULL;
		break;
	default:
		cerr << "error!" << endl;//there should be this edge
	}//find the assocated face and make it NULL.
	vvnode[i]->count--;
}
void Repair(vector<vector<VertexNode*> > &vnodeList, Face *f){
	/*when it comes to an contradiction, call this function to correct the face's vertices' order
		the computational complexity is O(1), since DeleteVnode, Swap, AddNode is O(1), and they are 
		called constant times
	*/
	for(int i=0;i<2;i++)
		DeleteVnode(f->ver[i], f->ver[i+1], vnodeList, f);
	DeleteVnode(f->ver[2], f->ver[0], vnodeList, f);

	Swap(f->ver[0], f->ver[1]);
	for(int i=0;i<2;i++)
		AddVnode(f->ver[i], f->ver[i+1], vnodeList, f);
	AddVnode(f->ver[2], f->ver[0], vnodeList, f);

}
void CheckEdge(Vertex *v0, Vertex *v1, vector<vector<VertexNode*> > &vnodeList, Face* f, queue<Face *> &qf){
	/*Check the edge(v0, v1), if there is a contradiction, call Repair to correct it,
		and find it's neighboring face and put it in queue
		since the operations below is local, the complexity is O(1)
	*/
	vector<VertexNode *> &vvnode =  vnodeList[v0->vid];
	int i=0;
	for (;i<vvnode.size();i++){
		if(vvnode[i]->v->vid == v1->vid)
			break;
	}// find edge(v0, v1)
	assert(i<vvnode.size());
	Face *f0;
	switch (vvnode[i]->count)
	{
	case 2://edge(v0, v1) appears twice, these should be a contradiction
		if (vvnode[i]->f[0]->fid==f->fid){
			f0 = vvnode[i]->f[1];
		}else{
			f0 = vvnode[i]->f[0];
		}//find f's neighbor sharing edge(v0, v1)
		if (!f0->visited){
			Repair(vnodeList, f0);
			qf.push(f0);
		}
		break;
	case 1: //edge(v0, v1) appears once, it could be a boundary or orindary edge 
		vector<VertexNode *> &vvnode =  vnodeList[v1->vid];
		int j=0;
		for (;j<vvnode.size();j++){
			if(vvnode[j]->v->vid == v0->vid)
				break;
		}// find edge (v1, v0)
		if(j<vvnode.size() && vvnode[j]->count>0){ // it is not a boundary
			if(vvnode[j]->f[0]!=NULL)
				f0 = vvnode[j]->f[0];
			else if(vvnode[j]->f[1]!=NULL)
				f0 = vvnode[j]->f[1];
			assert(vvnode[j]->count == 1);
			if(!f0->visited)
				qf.push(f0);
		}
		break;
	//otherwise it is a boundary edge
	}
}
int main(int argc, char **argv)
{
	if(argc <3){
		cout << "format: CheckMesh [in.obj] [out.m]" << endl;
		return 0;
	}
	ifstream ifile(argv[1]);
	ofstream ofile(argv[2]);

	string line, temp;
	int vsize=0;
	vector<Vertex *> verList;
	while (getline(ifile, line))
	{

		if(line.find("v")!=line.npos)
		{
			Vertex *v = new Vertex(vsize);
			verList.push_back(v);
			vsize++;
			ofile << "Vertex " << vsize << " " << line.substr(2, -1) << endl;
		}else
			break;
	}// process vertices, O(V)
	int fsize=0;
	vector< Face* > facelist;
	vector<vector<VertexNode *> > vnodeList; //local edges structure
	vector<VertexNode *> vn; //local edges of one vertex as its starting point
	for(int i=0;i<vsize;i++)
		vnodeList.push_back(vn);
	int vid[3];
	do
	{
		if (line.find("f")!=line.npos)
		{
			istringstream iss(line);
			iss >> temp >> vid[0] >> vid[1] >> vid[2];
			Vertex *v[3];
			for(int i=0;i<3;i++)
				v[i] = verList[vid[i] - 1];
			Face *f = new Face(fsize, v);
			for(int i=0;i<2;i++)//add edges
				AddVnode(v[i], v[i+1], vnodeList, f);//O(1)		
			AddVnode(v[2], v[0], vnodeList, f);
			
			facelist.push_back(f);
			++fsize;
		}
	}while (getline(ifile, line));//process faces, O(F)
	ifile.close();

	queue<Face*> qf;
	//Inital
	Face *f0 = facelist[0];	
	qf.push(f0);
	int count = 0;

	//loop invariant: every edge in the queue has at most one face visited
	while (!qf.empty()){
		Face *f = qf.front();
		f->visited = true;
		
		for(int i=0;i<2;i++)
			CheckEdge(f->ver[i], f->ver[i+1], vnodeList, f, qf);//O(1)
		CheckEdge(f->ver[2], f->ver[0], vnodeList, f, qf);
		qf.pop();	
	}//complexity O(F)

	for(int i=0;i<facelist.size();i++){
		ofile << "Face " << facelist[i]->fid+1 << " " << facelist[i]->ver[0]->vid+1 << " " << facelist[i]->ver[1]->vid+1 << " " << facelist[i]->ver[2]->vid+1 << endl;
	}//O(F)
	ofile.close();
	for(int i=0;i<facelist.size();i++)
	{
		delete facelist[i];
	}//O(F)
	for(int i=0;i<verList.size();i++)
		delete verList[i];
	//O(V)

	//total complexity O(V+F), since every face is associated to constant vertices(in this case, 3), therefore,
	//overall complexity is O(V)
	return 0;
}