#include "Application.h"
#include <FL/Fl_File_Chooser.H>
#include "GUI/gui.h"
#include <iostream>
#include "Core_Mesh/Iterators.h"
#include "Core_Mesh/Curve.h"
#include "Core_Mesh/TraitParser.h"

using namespace XMeshLib;

extern Gui *gui;

// the constructor method for the Application class
Application::Application()
{
	sMesh = NULL;
	tMesh = NULL;
	vec_B1 = NULL;
	vec_B2 = NULL;
	vec_U = NULL;
	vec_V = NULL;
	vec_w = NULL;
	vec_X1 = NULL;
	vec_X2 = NULL;
	indexvb = NULL;
	indexvin = NULL;
	boundaryv = NULL;
	mat_A = NULL;
	mat_v = NULL;
	NeighLam.clear();
	Lamda.clear();
}

// the method that gets executed when the readFile callback is called
void Application::ReadSourceMesh()
{

	// launch file browser
	char *file = fl_file_chooser("Pick a file from this list:", "*.m", "");
	if(file == NULL)
	return;

	std::cout << "Reading Source Mesh: " << file << "..." <<std::endl;
	if (sMesh)
	{
		delete sMesh;
	}
	sMesh = new Mesh;
	sMesh->read(file);
	std::cout << "Read SourceMesh [Done] " <<std::endl;	
}

void Application::ReadTargetMesh()
{

	// launch file browser
	char *file = fl_file_chooser("Pick a file from this list:", "*.m", "");
	if(file == NULL)
		return;

	std::cout << "Reading Target Mesh: " << file << "..." <<std::endl;
	if (tMesh)
	{
		delete tMesh;
	}

	tMesh = new Mesh;
	tMesh->read(file);
	std::cout << "Read Target Mesh [Done] " <<std::endl;	
}

// add application routines here:
void Application::WriteMesh()
{
	;
}




void Application::Init()
{
	int i;
	for (i=0;i<=vn;i++)
		boundaryv[i] =0;
	for (i=0;i<nb;i++)
		indexvb[i] = 0;
	for (i=0;i<inp;i++)
		indexvin[i] = 0;
}

void Application::ClearMapping()
{
	if (vec_B1)
	{
		delete [] vec_B1;
		vec_B1 = NULL;
	}
	if (vec_B2)
	{
		delete [] vec_B2;
		vec_B2 = NULL;
	}
	if (vec_U)
	{
		delete [] vec_U;
		vec_U = NULL;
	}
	if (vec_V)
	{
		delete [] vec_V;
		vec_V = NULL;
	}
	if (vec_X1)
	{
		delete [] vec_X1;
		vec_X1 = NULL;
	}
	if (vec_X2)
	{
		delete [] vec_X2;
		vec_X2 = NULL;
	}

	if (indexvb)
	{
		delete [] indexvb;
		indexvb = NULL;
	}
	if (indexvin)
	{
		delete [] indexvin;
		indexvin = NULL;
	}
	if (boundaryv)
	{
		delete [] boundaryv;
		boundaryv = NULL;
	}
	if (vec_w)
	{
		delete [] vec_w;
		vec_w = NULL;
	}

	if (mat_v)
	{
		for (int i=0;i<inp;i++)
		{
			delete []mat_v[i];
			mat_v [i] = NULL;
		}
		delete [] mat_v;
		mat_v = NULL;
	}


	if (mat_A)
	{
		for (int i=0;i<inp;i++)
		{
			delete []mat_A[i];
			mat_A [i] = NULL;
		}
		delete [] mat_A;
		mat_A = NULL;
	}

	for (int i=0;i<NeighLam.size();i++)
		NeighLam[i].clear();
	NeighLam.clear();

	for (int i=0;i<Lamda.size();i++)
		Lamda[i].clear();
	Lamda.clear();


}

//we have theMesh  get its uv mapping
void Application::HarmMap(Mesh *theMesh)
{
	std::cout<<"Discrete Harmonic Mapping: START...."<<std::endl;

	this->ClearMapping ();  //clear vec_U,vec_V,vec_X1,vec_X2,mat_A

	if (theMesh==NULL)  return;

	double nbr = 1;  //the ratio of boundary number as boundary condition
	
	//we trace to get the boundary of theMesh
	MeshVertexIterator viter(theMesh);
	for ( ; !viter.end(); viter++)
		if ((*viter)->boundary())
			break;
	Curve  ct;
	ct.CreateBoundaryLoop (*viter);  //star from one boundary vertex to trace the boundary loop store in the ct.helist, we could just ct[i] to direct ct.helist[i]

	int nsize = ct.heSize();  //the total vertex number of the boundary
	nb = nsize * nbr;  //the boundary points    
	
	int deltan = 1.0/nbr;  //uniformly to have the boundary points, step is deltan, every deltan to get one
    double theta = 0;
	//double deltath = 2*PI/(nb*1.0);
	double deltath = 360.0/(nb*1.0);

	vn = theMesh->numVertices();
	inp = vn - nb; //the interior points = the total points - the boundary points

	vec_U = new float [nb];  //boundary points #
	vec_V = new float [nb];
	vec_B1 = new float [inp];
	vec_B2 = new float [inp];
	vec_X1 = new float [inp]; //interior points #
	vec_X2 = new float [inp];

	indexvin = new int [inp]; //interior points
	indexvb = new int [nb]; //boundary points
	boundaryv = new bool [vn+1]; //all the points
    this->Init();  //initiate the indexv =0, bounaryv = 0
	
	int i,j=0;  //we set the boundary mapping in here
	for (i=0;i<nsize;i+=deltan)
	{  //the current object boundary vertex is ct[i]->target();

		int vid = ct[i]->target()->id();
		boundaryv[vid] = true;    //build the boundary index 
        indexvb [j] = vid;   

		vec_U[j] = sin(theta*PI/180.0);  //map the boundary to the unite circle
		vec_V[j] = cos(theta*PI/180.0);
		j++;
		theta = theta + deltath;
	}

	//pack the interior points to the index 1...inp

	j=0;
	for (i=1;i<=vn;i++)  
	{
		if (boundaryv[i]==true)  continue;
		indexvin[j] = i;
		j++;
	}


	this->ComputeLamda (theMesh);
	this->ComputeCoefMatrix ();  //get the matrix A


	//now we have A and B, to solve get vec_X1, vec_X2 as the interior points mapping u,v

	//need to pay attention to the indexv value....
    this->SVDCoefMatrix_Nrcpp();

	XMeshLib_Nrcpp::svbksb (mat_A,vec_w,mat_v,inp,inp,vec_B1,vec_X1);
	XMeshLib_Nrcpp::svbksb (mat_A,vec_w,mat_v,inp,inp,vec_B2,vec_X2);


	//so, now we get the interior points uv as vec_X1,vec_X2
    for (i=0;i<inp;i++)
	{
		int vid = indexvin[i];
		Vertex *v = theMesh->idVertex(vid);

		std::string keytype = "uv";
    	double uvs[2]={vec_X1[i],vec_X2[i]};
		TraitParser::UpdateTrait(v->string(),keytype,2,uvs);	
	
	}
	for (i=0;i<nb;i++)
	{
		int vid = indexvb[i];
		Vertex *v = theMesh->idVertex(vid);

		std::string keytype = "uv";
		double uvs[2]={vec_U[i],vec_V[i]};
		TraitParser::UpdateTrait(v->string(),keytype,2,uvs);	
	}
	theMesh->write("UV.m");


	std::cout<<"Discrete Harmonic Mapping [DONE]!"<<std::endl;
}




void Application::SVDCoefMatrix_Nrcpp()
{	
	mat_v = new xreal * [inp];
	vec_w = new xreal [inp];
	int i;
	for (i=0;i<inp; ++i)
		mat_v[i] = new xreal[inp];

	XMeshLib_Nrcpp::svdcmp(mat_A, inp, inp, vec_w, mat_v);
	XMeshLib_Nrcpp::preparesvbksb(vec_w, inp);
}


double Application::CornerAngle (XMeshLib::HalfEdge * he)
{
	Point &p0 = he->target()->point();
	Point &p2 = he->source()->point();
	Point &p1 = he->he_next()->target()->point();

	double a = (p1-p0).norm();
	double b = (p0-p2).norm();
	double c = (p2-p1).norm();

	double costheta = (a*a+b*b-c*c)/(2*a*b);
	double theta = acos (costheta);

	return theta;

}

double Application::CalcuateL (Mesh*theMesh, int i,int j)
{
	Edge *e = theMesh->idEdge(i,j);
	if (e->boundary())
	{
		HalfEdge *he1 = e->halfedge(0)->he_next();
		double corangle = CornerAngle (he1);
		return (1/tan(corangle))/2;
	}
	else
	{
		HalfEdge *he1 = e->halfedge(0)->he_next();
		HalfEdge *he2 = e->halfedge(1)->he_next();
		double corangle1 = CornerAngle (he1);
		double corangle2 = CornerAngle (he2);
		return ((1/tan(corangle1)+1/tan(corangle2)))/2;
	}
}

void Application::ComputeLamda(Mesh *theMesh)
{
	int i,j;
	for (i=0;i<=vn;i++)
	{
		std::vector<float> t;
		Lamda.push_back(t);
		std::vector<int> t2;
		NeighLam.push_back(t2);
	}

	for (i=1;i<=vn;i++)  //for the vertex i, to get its neighbor's Lamda
	{
		Vertex *v = theMesh->idVertex(i);

		HalfEdge *he = v->halfedge();
		HalfEdge *the = he;

		while (true)
		{
			j = the->source()->id();
			if (!Neighbor(i,j))  //if he not already in there
			{
                double tlam=CalcuateL (theMesh,i,j);
				NeighLam[i].push_back(j);  //set j as i's neighborhoods
				Lamda[i].push_back(tlam);
				NeighLam[j].push_back(i);
				Lamda[j].push_back(tlam);
			}
			the = the->he_next()->he_sym();

			if (the==NULL || the==he) break;
		}

		if (the==NULL)  //that means v is the boundary point
		{
			if (v->boundary() == false)  printf ("Error: boundary Neighbor judges!\n");

			the = he->he_sym();
            if (the!=NULL)
			{
				the = the->he_prev();
				while (the!=NULL && the!=he)
				{
					j = the->source()->id();
					if (!Neighbor(i,j))
					{
						double tlam = CalcuateL(theMesh,i,j);
						NeighLam[i].push_back(j);
						Lamda[i].push_back(tlam);
						NeighLam[j].push_back(i);
						Lamda[j].push_back(tlam);
					}
					the = the->he_sym();
					if (the==NULL) break;
					the = the->he_prev();			
				}
			}
		}
	}

	//we need the normalize about the lamda
	for (i=1;i<=vn;i++)
	{
		double s = 0;

		for (j=0;j<Lamda[i].size();j++)
			s = s + Lamda[i][j];

		for (j=0;j<Lamda[i].size();j++)
			Lamda[i][j] = Lamda[i][j]/s;
	}
}


bool Application::Neighbor (int i,int j)
{
	int k;
	for (k=0;k<NeighLam[i].size();k++)
		if (NeighLam[i][k]==j)
			return true;
	return false;
}

float Application::GetLamda (int i,int j)
{
	int k;
	for (k=0;k<NeighLam[i].size();k++)
		if (NeighLam[i][k]==j)
			break;
	return Lamda[i][k];
}


// we compute the matrix A and the corresponding b in here.
void Application::ComputeCoefMatrix()
{
	mat_A = new float *[inp];


	int i,j;
	for (i=0;i<inp;i++)
	{
		mat_A [i] = new float [inp];
		int id = indexvin[i];

		for (j=0;j<inp;j++)
		{
			int jd = indexvin[j];

			if (id==jd) mat_A[i][j] = 1;
			else if (Neighbor(id,jd))  //j is in i's neighbor, then return true
			{
				mat_A[i][j] = (-1)*GetLamda(id,jd);  // GetLamda (id_i,j)

			}
			else mat_A [i][j] = 0;
		}
	}


	for (i=0;i<inp;i++)  //we calculate the 
	{
		vec_B1 [i] = 0;
		vec_B2 [i] = 0;

		int id = indexvin[i];

		for (j=1;j<=vn;j++)
		{
			if (boundaryv[j]==true && Neighbor(id,j))   //here, i is equal to the id Neighbor (id_i, j)
			{
				vec_B1 [i] += GetLamda(id,j) * vec_U[FIndindex(j)];  // GetLamda (id_i,j)
				vec_B2 [i] += GetLamda(id,j) * vec_V[FIndindex(j)];  // GetLamda (id_i,j)


			}
		}	
		
	}

}


int Application::FIndindex (int j)  //find the indexvb[i] = j
{
	int i;
	for (i=0;i<nb;i++)
		if (indexvb[i]==j)
			break;
	return i;
}

int Application::TestPointTriangle (Point p, Point a, Point b, Point c,double bary[3])
{
	double epsilon = 1e-9;
	double nu = ((b-p)^(c-b)).norm();
	double nv = ((c-p)^(a-c)).norm();
	double nw = ((a-p)^(b-a)).norm();

	double m = ((b-a)^(c-b)).norm();
    
	bary[0] = nu/m;
	bary[1] = nv/m;
	bary[2] = nw/m;

	return (fabs(bary[0]+bary[1]+bary[2]-1)<=epsilon);
}

//do the point location, output should be the  ioface, bary[3] 
// to indicate the puv belongs to ioface, and with coefficient bary[3] (in tMesh)
int Application::PointLocation (Face *& face,double bary[3], Point puv)
{
	Point wpuv [3];
	int flag = 0;

	for (MeshFaceIterator filter(tMesh); !filter.end();++filter)
	{
		face = *filter;

		int i = 0;
		for (FaceVertexIterator viter(face); !viter.end(); ++viter)
		{
			Vertex *ver = *viter;

			double val[2];
			TraitParser::ReadTrait(ver->string(),"uv",2,val);
			wpuv[i][0]= val[0];
			wpuv[i][1]= val[1];
			wpuv[i][2]= 0;
			i++;
		}
		if (TestPointTriangle(puv,wpuv[0],wpuv[1],wpuv[2],bary))
		{
			flag  = 1;
			break;
		}
	}

	return flag;

}

//for the uv-point to get its face location in tmesh (uv-plane) and the corresponding 
// vertex baycentry coordinates
void Application::Baycentry (Point puv,Point &nvp, Point &nvpnormal,double rgb[3],bool bf)  //boundary flag 0 not boundary, 1 boundary
{
	Face *ioface=NULL;
	double bary[3];
	int flag = PointLocation (ioface,bary,puv);


	std::string keytype = "rgb";

	if (flag ==0 && bf)    //we find the nearest boundary point
	{

		double val[2];
		int mindx =0;
		double mindis = 1e5;
		for (int i=0;i<ctb->heSize();i++)
		{
			Vertex *vs = ctb->VertexAt(i);
			TraitParser::ReadTrait(vs->string(),"uv",2,val);
		    Point p (val[0],val[1],0);
			double dis = (puv-p).norm();
			if (dis<mindis)
			{
				mindis = dis;
				mindx = i;
			}
		}
		nvp = ctb->VertexAt(mindx)->point();
		nvpnormal = tMesh->verNormal(ctb->VertexAt(mindx));
		TraitParser::ReadTrait(ctb->VertexAt(mindx)->string(),keytype,3,rgb);
		return ;
	}

	Vertex *v1 = ioface->halfedge()->target();
	Vertex *v3 = ioface->halfedge()->source();
	Vertex *v2 = ioface->halfedge()->he_prev()->source();

	Point fv1 = tMesh->idVertex(v1->id())->point();
	Point fv2 = tMesh->idVertex(v2->id())->point();
	Point fv3 = tMesh->idVertex(v3->id())->point();

	nvp = fv1*bary[0] + fv2 *bary[1] + fv3*bary[2];

	Point nfv1 = tMesh ->verNormal(v1);
	Point nfv2 = tMesh ->verNormal(v2);
	Point nfv3 = tMesh ->verNormal(v3);

	nvpnormal = nfv1*bary[0] + nfv2 *bary[1] + nfv3 * bary[2];

	double rgb1[3];
	double rgb2[3];
	double rgb3[3];
	TraitParser::ReadTrait(v1->string(),keytype,3,rgb1);
	TraitParser::ReadTrait(v2->string(),keytype,3,rgb2);
	TraitParser::ReadTrait(v3->string(),keytype,3,rgb3);
	rgb[0] = rgb1[0]*bary[0] + rgb2[0]*bary[1] + rgb3[0] * bary[2];
	rgb[1] = rgb1[1]*bary[0] + rgb2[1]*bary[1] + rgb3[1] * bary[2];
	rgb[2] = rgb1[2]*bary[0] + rgb2[2]*bary[1] + rgb3[2] * bary[2];

}


void Application::Morphing ()
{
	std::cout<<"Inter-Object Morphing: START...."<<std::endl;

	//we create the boundary curve of the target object
	ctb = new Curve ();
	MeshVertexIterator  vitert (tMesh);
	for (;!vitert.end();++vitert)
		if ((*vitert)->boundary()) break;

	ctb->CreateBoundaryLoop(*vitert);


	//now we have the sMesh and tMesh with their uv
	MeshVertexIterator viters (sMesh);

	std::string keytype = "Opos";
	std::string keytype2 = "Onormal";
	std::string keytype3 = "normal";
	std::string keytype4 = "Orgb";
	std::string keytype5 = "rgb";

	//we are going to find the oppos for each vertex in source mesh
	for (;!viters.end();++viters)
	{
		Vertex *vs = *viters;
		double val[2];
		TraitParser::ReadTrait(vs->string(),"uv",2,val);
		Point puv (val[0],val[1],0);
		Point nvp;  //the oppos need to find out
		Point nvpnormal;
		double rgb[3];

		Baycentry (puv,nvp,nvpnormal,rgb,vs->boundary());

		double oppos[3];
		oppos[0] = nvp[0];
		oppos[1] = nvp[1];
		oppos[2] = nvp[2];
		TraitParser::UpdateTrait(vs->string(),keytype,3,oppos);

		oppos[0] = nvpnormal[0];
		oppos[1] = nvpnormal[1];
		oppos[2] = nvpnormal[2];
		TraitParser::UpdateTrait(vs->string(),keytype2,3,oppos);

		TraitParser::UpdateTrait(vs->string(),keytype4,3,rgb);

	}
	std::cout<<"Inter-Object Morphing: [DONE]!"<<std::endl;
	sMesh->write ("Morphing.m");

	//now smesh has vertex position and its oppos, also with normal and onormal
	double delta = 0.1;
	double t = 0;

	char str[10];

	Mesh *tmpms = new Mesh;
	sMesh->copyTo(*tmpms);

	int i=1;
	for (t=0;t<1.-0.000005;t=t+delta)  //each time, we generate one temple .m file
	{
		MeshVertexIterator viter (tmpms);

		for (;!viter.end();++viter)
		{
			Vertex *v = *viter;
			double oppos[3];  //read the oppos
			double opposn [3]; //read the onnormal
			TraitParser::ReadTrait(v->string(),keytype,3,oppos);
			TraitParser::ReadTrait(v->string(),keytype2,3,opposn);
            Point opos(oppos[0],oppos[1],oppos[2]);
			Point oposn (opposn[0],opposn[1],opposn[2]);

			Point p = v->point();
			double pnd[3];
			TraitParser::ReadTrait(v->string(),keytype3,3,pnd);
			Point pn(pnd[0],pnd[1],pnd[2]); // = sMesh->verNormal(v);

			Point tp = p*(1-t) + opos*t;
			Point tpn = pn *(1-t) + oposn *t;
			double tpnd[3];
			tpnd[0] = tpn[0]; tpnd[1] = tpn[1]; tpnd[2] = tpn[2];

			v->point() = tp;
			TraitParser::UpdateTrait(v->string(),keytype3,3,tpnd);

			double rgb0[3];
			double rgb1[3];
			double rgb[3];
			TraitParser::ReadTrait(v->string(),keytype5,3,rgb0);
			TraitParser::ReadTrait(v->string(),keytype4,3,rgb1);
			rgb[0] = rgb0[0]*(1-t) + rgb1[0]*t;
			rgb[1] = rgb0[1]*(1-t) + rgb1[1]*t;
			rgb[2] = rgb0[2]*(1-t) + rgb1[2]*t;
			TraitParser::UpdateTrait(v->string(),keytype5,3,rgb);

		}

        sprintf (str,"%d.m",i++);
	    tmpms->write(str);		
	}
	delete tmpms;
	delete ctb;
}


