#ifndef _XMESHLIB_POINT2_H_
#define _XMESHLIB_POINT2_H_

#include <iostream>
#include <cassert>
#include <cmath>

namespace XMeshLib
{

	template <class _T> class Point2;

	template <class _T>
	std::istream& operator >> (std::istream &is, Point2<_T> &p) 
	{
		is >> p[0] >> p[1];
		return is;
	}

	template <class _T>
	std::ostream& operator << (std::ostream &os, const Point2<_T> &p) 
	{
		return os << p.X() << ' ' << p.Y();
	}


	template <class _T>
	class Point2
	{

	public:

		Point2(const _T &x, const _T &y) { v[0] = x; v[1] = y; }
		Point2() { v[0] = v[1] = (_T)(0.0); }
		Point2(const Point2 &p) { v[0] = p[0]; v[1] = p[1]; }
		~Point2() {}

		_T X() const { return v[0]; }
		_T Y() const { return v[1]; }

		_T & operator[](int i) { assert(0 <= i && i < 2 ); return v[i]; }	
		_T operator[](int i) const { assert(0 <= i && i < 2); return v[i]; }

		_T norm2() const { return fabs(v[0] * v[0] + v[1] * v[1]); }
		_T norm() const { return sqrt(fabs(v[0] * v[0] + v[1] * v[1])); }

		Point2 & operator += (const Point2 & p) { v[0] += p[0]; v[1] += p[1]; return *this; }
		Point2 & operator -= (const Point2 & p) { v[0] -= p[0]; v[1] -= p[1]; return *this; }
		Point2 & operator *= (const _T &s) { v[0] *= s; v[1] *= s; return *this; }
		Point2 & operator /= (const _T &s) { v[0] /= s; v[1] /= s; return *this; }

		_T operator *(const Point2 &p) const { return v[0]*p[0] + v[1]*p[1]; }

		Point2 operator +(const Point2 &p) const { Point2 r(v[0]+p[0], v[1]+p[1]); return r; }
		
		Point2 operator -(const Point2 &p) const { Point2 r(v[0]-p[0], v[1]-p[1]); return r; }
		
		Point2 operator *(const _T &s) const { Point2 r(v[0]*s, v[1]*s); return r; }

		Point2 operator /(const _T &s) const { Point2 r(v[0]/s, v[1]/s); return r; }

		_T operator ^(const Point2 &p) const { return v[0]*p[1] - v[1]*p[0]; }

		Point2 operator -() const { Point2 p(-v[0],-v[1]); return p; }

	private:
		_T v[2];
	};


	//--------------------------------------------
	typedef Point2<float>  Point2f;
	typedef Point2<double> Point2d;
	typedef Point2<int>    Point2i;

}

#endif //XMESHLIB_POINT2_H