//
// Name :         Texture.h
// Description :  Header for CTexture, texture image class for OpenGL.
// Version :      See Texture.cpp
//

#ifndef _XMESHLIB_TEXTURE_H
#define _XMESHLIB_TEXTURE_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <iostream>
#include <fstream>
#include <cassert>
#include <GL/glew.h>

//the following two including are commented out
//#include <GL/glu.h> 
//#include <GL/gl.h>
//because of the using of glew.h ---xav


#ifndef WIN32

typedef long				LONG;
typedef unsigned long       DWORD;
typedef int                 BOOL;
typedef unsigned char       BYTE;
typedef unsigned short      WORD;
typedef float               FLOAT;

/*	These structures may suffer from packing problems in UNIX
	One possible work-around is to use the "packed" attribute 
	against the members of a structure. i.e. add the following
	line after every member of a structure: "__attribute__((__packed__))"
*/
typedef struct tagRGBQUAD {
        BYTE    rgbBlue;
        BYTE    rgbGreen;
        BYTE    rgbRed;
        BYTE    rgbReserved;
} RGBQUAD;

typedef struct tagBITMAPFILEHEADER {
        WORD    bfType;
        DWORD   bfSize;
        WORD    bfReserved1;
        WORD    bfReserved2;
        DWORD   bfOffBits;
} BITMAPFILEHEADER, *LPBITMAPFILEHEADER, *PBITMAPFILEHEADER;

typedef struct tagBITMAPINFOHEADER{
        DWORD      biSize;
        LONG       biWidth;
        LONG       biHeight;
        WORD       biPlanes;
        WORD       biBitCount;
        DWORD      biCompression;
        DWORD      biSizeImage;
        LONG       biXPelsPerMeter;
        LONG       biYPelsPerMeter;
        DWORD      biClrUsed;
        DWORD      biClrImportant;
} BITMAPINFOHEADER, *LPBITMAPINFOHEADER, *PBITMAPINFOHEADER;

typedef struct tagBITMAPINFO {
    BITMAPINFOHEADER    bmiHeader;
    RGBQUAD             bmiColors[1];
} BITMAPINFO, *LPBITMAPINFO, *PBITMAPINFO;

#define RGB(r,g,b)          ((COLORREF)(((BYTE)(r)|((WORD)((BYTE)(g))<<8))|(((DWORD)(BYTE)(b))<<16)))
#define GetRValue(rgb)      ((BYTE)(rgb))
#define GetGValue(rgb)      ((BYTE)(((WORD)(rgb)) >> 8))
#define GetBValue(rgb)      ((BYTE)((rgb)>>16))


#else
#include <windows.h>
#endif


namespace XMeshLib
{

class CTexture  
{
public:
	GLuint MipTexName();
	CTexture();
	CTexture(const CTexture &p_img);
	virtual ~CTexture();

	GLuint TexName();

	void Fill(int r, int g, int b);
	void Set(int x, int y, int r, int g, int b);
	void SetSize(int p_x, int p_y);
	void SameSize(const CTexture &p_img);
	void Copy(const CTexture &p_img);
	bool LoadFile(const char * lpszPathName);
	bool Empty() const {return m_width <= 0 || m_height <= 0;}
	CTexture &operator=(const CTexture &p_img);

	unsigned char *operator[](int i) {return m_image[i];}
	const unsigned char *operator[](int i) const {return m_image[i];}
	unsigned char *Row(int i) {return m_image[i];}
	const unsigned char *Row(int i) const {return m_image[i];}

	int Width() const {return m_width;}
	int Height() const {return m_height;}
	unsigned char *ImageBits() const {return m_image[0];}

private:
	bool m_mipinitialized;
	bool m_initialized;
	GLuint m_texname;
	GLuint m_miptexname;
	bool ReadDIBFile(std::istream &file);
	bool ReadPPMFile(std::istream &file);
	int m_height;
	int m_width;
	unsigned char ** m_image;
};

}
#endif //_XMESHLIB_TEXTURE_H
